/*
 * SortButtonRenderer.java
 *
 * Copyright (c) 2004,2005 Gregory Kotsaftis
 * gregkotsaftis@yahoo.com
 * http://zeus-jscl.sourceforge.net/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package gr.zeus.ui.table.sort;

import java.awt.Component;
import java.awt.Insets;
import java.util.Hashtable;
import javax.swing.ImageIcon;
import javax.swing.table.TableCellRenderer;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.SwingConstants;

/**
 * Table header renderer for TableSorter. Original code from:
 * <a href="http://www.senun.com/Left/Programming/Java_old/Examples_swing/SwingExamples.html">
 * http://www.senun.com/Left/Programming/Java_old/Examples_swing/SwingExamples.html</a>
 * <br>Icons are from the Eclipse project:
 * <a href="http://www.eclipse.org">http://www.eclipse.org</a>
 * <p>
 * <p>
 * @author Gregory Kotsaftis
 * @since 1.02
 */
public final class SortButtonRenderer extends JButton
    implements TableCellRenderer {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = -2979027345440168434L;

    /**
     * No direction.
     */
    public static final int DIR_NONE = 0;

    /**
     * Down direction.
     */
    public static final int DIR_DOWN = 1;

    /**
     * Up direction.
     */
    public static final int DIR_UP   = 2;
    
    /**
     * Blank image icon.
     */
    public static final ImageIcon EMPTY_ICON = new ImageIcon( 
        SortButtonRenderer.class.getResource(
            "/gr/zeus/ui/table/sort/res/blank.gif"));

    /**
     * Up image icon 1.
     */
    public static final ImageIcon UP1_ICON = new ImageIcon( 
        SortButtonRenderer.class.getResource(
            "/gr/zeus/ui/table/sort/res/up1.gif"));

    /**
     * Down image icon 1.
     */
    public static final ImageIcon DOWN1_ICON = new ImageIcon( 
        SortButtonRenderer.class.getResource(
            "/gr/zeus/ui/table/sort/res/down1.gif"));

    /**
     * Up image icon 2.
     */
    public static final ImageIcon UP2_ICON = new ImageIcon( 
        SortButtonRenderer.class.getResource(
            "/gr/zeus/ui/table/sort/res/up2.gif"));

    /**
     * Down image icon 2.
     */
    public static final ImageIcon DOWN2_ICON = new ImageIcon( 
        SortButtonRenderer.class.getResource(
            "/gr/zeus/ui/table/sort/res/down2.gif"));

    /**
     * Up image icon 3.
     */
    public static final ImageIcon UP3_ICON = new ImageIcon( 
        SortButtonRenderer.class.getResource(
            "/gr/zeus/ui/table/sort/res/up3.gif"));

    /**
     * Down image icon 3.
     */
    public static final ImageIcon DOWN3_ICON = new ImageIcon( 
        SortButtonRenderer.class.getResource(
            "/gr/zeus/ui/table/sort/res/down3.gif"));
    
    /**
     * Selected column to sort.
     */
    private int m_pushedColumn;

    /**
     * State <code>Hashtable</code>.
     */
    private Hashtable<Integer, Integer> m_state;

    /**
     * Down button.
     */
    private JButton m_downButton;

    /**
     * Up button.
     */
    private JButton m_upButton;

    
    /**
     * Constructor.
     */
    public SortButtonRenderer()
    {
        m_pushedColumn = -1;
        m_state = new Hashtable<Integer, Integer>();
    
        setMargin( new Insets(0,0,0,0) );
        setHorizontalTextPosition( SwingConstants.LEFT );
        setIcon( new BlankIcon() );
    
        // perplexed  
        // ArrowIcon(SwingConstants.SOUTH, true)   
        // BevelArrowIcon(int direction, boolean isRaisedView, 
        //   boolean isPressedView)
    
        m_downButton = new JButton();
        m_downButton.setMargin( new Insets(0,0,0,0) );
        m_downButton.setHorizontalTextPosition( SwingConstants.LEFT );
        m_downButton.setIcon( new BevelArrowIcon(DIR_DOWN, false, false) );
        m_downButton.setPressedIcon( new BevelArrowIcon(DIR_DOWN, false, true) );
    
        m_upButton = new JButton();
        m_upButton.setMargin( new Insets(0,0,0,0) );
        m_upButton.setHorizontalTextPosition( SwingConstants.LEFT );
        m_upButton.setIcon( new BevelArrowIcon(DIR_UP, false, false) );
        m_upButton.setPressedIcon( new BevelArrowIcon(DIR_UP, false, true) );
    }

    
    /**
     * Constructor.
     * <p>
     * @param blank     The blank <code>ImageIcon</code>.
     * @param up        The up <code>ImageIcon</code>.
     * @param down      The down <code>ImageIcon</code>.
     */
    public SortButtonRenderer(ImageIcon blank, ImageIcon up, ImageIcon down)
    {
        m_pushedColumn = -1;
        m_state = new Hashtable<Integer, Integer>();
    
        setMargin( new Insets(0,0,0,0) );
        setHorizontalTextPosition( SwingConstants.LEFT );
        setIcon( blank );
    
        m_upButton = new JButton();
        m_upButton.setMargin( new Insets(0,0,0,0) );
        m_upButton.setHorizontalTextPosition( SwingConstants.LEFT );
        m_upButton.setIcon( up );
        m_upButton.setPressedIcon( up );

        m_downButton = new JButton();
        m_downButton.setMargin( new Insets(0,0,0,0) );
        m_downButton.setHorizontalTextPosition( SwingConstants.LEFT );
        m_downButton.setIcon( down );
        m_downButton.setPressedIcon( down );
    }
    
    
    /**
     * Gets the table cell renderer.
     * <p>
     * @param table         The <code>JTable</code>.
     * @param value         The value <code>Object</code>.
     * @param isSelected    <code>true</code> if selected.
     * @param hasFocus      <code>true</code> if has focus.
     * @param row           The row number.
     * @param column        The column number.
     * <p>
     * @return              The table cell renderer.
     */
    public Component getTableCellRendererComponent(JTable table, Object value,
        boolean isSelected, boolean hasFocus, int row, int column)
    {
        JButton button = this;
        Object obj = m_state.get( new Integer(column) );
        
        if( obj != null )
        {
            if( ((Integer)obj).intValue() == DIR_DOWN )
            {
                button = m_downButton;
            }
            else
            {
                button = m_upButton;
            }
        }

        button.setText( (value ==null) ? "" : value.toString() );
        boolean isPressed = (column == m_pushedColumn);
        button.getModel().setPressed( isPressed );
        button.getModel().setArmed( isPressed );
        
        return( button );
    }

    
    /**
     * Sets the pressed column.
     * <p>
     * @param col   The column number.
     */
    public void setPressedColumn(int col)
    {
        m_pushedColumn = col;
    }

    
    /**
     * Sets the selected column.
     * <p>
     * @param col   The column number.
     */
    public void setSelectedColumn(int col)
    {
        if( col < 0 )
        {
            return;
        }
        
        Integer value = null;
        Object obj = m_state.get( new Integer(col) );
        
        if( obj == null )
        {
            value = new Integer( DIR_DOWN );
        }
        else
        {
            if( ((Integer)obj).intValue() == DIR_DOWN )
            {
                value = new Integer( DIR_UP );
            }
            else
            {
                value = new Integer( DIR_DOWN );
            }
        }

        m_state.clear();
        m_state.put( new Integer(col), value );
    }

    
    /**
     * Gets the button's state.
     * <p>
     * @param col   The column number.
     * <p>
     * @return      The state.
     */
    public int getState(int col)
    {
        int retValue;
        Object obj = m_state.get( new Integer(col) );
        
        if( obj == null )
        {
            retValue = DIR_NONE;
        }
        else
        {
            if( ((Integer)obj).intValue() == DIR_DOWN )
            {
                retValue = DIR_DOWN;
            }
            else
            {
                retValue = DIR_UP;
            }
        }

        return( retValue );
    }
    
}
