/*
 * SortableTableModel.java
 *
 * Copyright (c) 2004,2005 Gregory Kotsaftis
 * gregkotsaftis@yahoo.com
 * http://zeus-jscl.sourceforge.net/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package gr.zeus.ui.table.sort;

import java.util.Vector;
import javax.swing.table.DefaultTableModel;

/**
 * Table model for TableSorter. Original code from:
 * <a href="http://www.senun.com/Left/Programming/Java_old/Examples_swing/SwingExamples.html">
 * http://www.senun.com/Left/Programming/Java_old/Examples_swing/SwingExamples.html</a>
 * <p>
 * @author Gregory Kotsaftis
 * @since 1.02
 */
public class SortableTableModel extends DefaultTableModel {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 4088661679003225474L;

    /**
     * Row indexes.
     */
    private int[] m_indexes;

    /**
     * Instance of TableSorter.
     */
    private transient TableSorter m_sorter;

    
    /**
     * Constructor.
     */
    public SortableTableModel()
    {
        super();
    }

    
    /**
     * Constructor.
     * <p>
     * @param rowCount      The row count.
     * @param columnCount   The column count.
     */
    public SortableTableModel(int rowCount, int columnCount)
    {
        super(rowCount, columnCount);
    }
    
    
    /**
     * Constructor.
     * <p>
     * @param data          The array of data.
     * @param columnNames   The array of column names.
     */
    public SortableTableModel(Object[][] data, Object[] columnNames)
    {
        super(data, columnNames);
    }

    
    /**
     * Constructor.
     * <p>
     * @param columnNames   The array of column names.
     * @param rowCount      The row count.
     */
    public SortableTableModel(Object[] columnNames, int rowCount)
    {
        super(columnNames, rowCount);
    }
    
    
    /**
     * Constructor.
     * <p>
     * @param columnNames   The <code>Vector</code> with the column names.
     * @param rowCount      The row count.
     */
    public SortableTableModel(Vector columnNames, int rowCount)
    {
        super(columnNames, rowCount);
    }
    
    
    /**
     * Constructor.
     * <p>
     * @param data          The <code>Vector</code> with  data.
     * @param columnNames   The <code>Vector</code> with the column names.
     */
    public SortableTableModel(Vector data, Vector columnNames)
    {
        super(data, columnNames);
    }
    
    
    /**
     * Gets the value at the specified cell.
     * <p>
     * @param row   The row number.
     * @param col   The column number.
     * @return      The value at the cell.
     */
    public Object getValueAt(int row, int col)
    {
        int rowIndex = row;
        if( m_indexes != null )
        {
            rowIndex = m_indexes[row];
        }

        return( super.getValueAt(rowIndex, col) );
    }

    
    /**
     * Sets the value at the specified cell.
     * <p>
     * @param value     The value to set.
     * @param row       The row number.
     * @param col       The column number.
     */
    public void setValueAt(Object value, int row, int col)
    {
        int rowIndex = row;
        if( m_indexes != null )
        {
            rowIndex = m_indexes[row];
        }
    
        super.setValueAt(value, rowIndex, col);
    }

    
    /**
     * Sorts the table by column.
     * <p>
     * @param column    The column number.
     * @param isAscent  Check for ascending or decending sort.
     */
    public void sortByColumn(int column, boolean isAscent)
    {
        if( m_sorter == null )
        {
            m_sorter = new TableSorter(this);
        }
        
        m_sorter.sort(column, isAscent);   
        fireTableDataChanged();
    }

    
    /**
     * Gets row indexes.
     * <p>
     * @return  Array of indexes.
     */
    public int[] getIndexes()
    {
        int n = getRowCount();
        
        if( m_indexes != null )
        {
            if( m_indexes.length == n )
            {
                return( m_indexes );
            }
        }

        m_indexes = new int[n];
        
        for(int i=0; i<n; i++)
        {
            m_indexes[i] = i;
        }

        return( m_indexes );
    }
    
}
